/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Rob Jansen
 *
 * File:
 *    pwm.h
 *
 * Description:
 *    PWM defines and functions
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include <lpc2xxx.h>
#include <config.h>
#include <pwm.h>

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

/*****************************************************************************
 *
 * Description:
 *    Initialize the PWM unit to generate a variable duty cycle signal on
 *    PWM2. Connect signal PWM2 to pin P0.7.
 *    The function sets initial frequency. Initial duty cucle is set to 0%.
 *
 * Params:
 *    [in] initialFreqValue - the initial frequency value in Hz.
 *                            This value calculated as:
 *
 *                            (crystal frequency * PLL multiplication factor)
 *                            -----------------------------------------------
 *                                 (VPBDIV factor * desired frequency)
 *
 * ToDo:
 *    - Add dynamic frequency scaling
 ****************************************************************************/
void pwmInit(unsigned int initialFreqValue)
{
	unsigned int match;
	/*
	 * initialize PWM
	 */

	match = ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (VPBDIV_FACTOR * initialFreqValue));

	PWM_PR  = 0x00000000;                                  //set prescale to 0
	PWM_MCR = PWM_MCR_PWMMR0R;                             //counter resets on MR0 match (period time)
	PWM_MR0 = match;                                       //MR0 = period cycle time
	PWM_MR2 = 0;                                           //MR2 = duty cycle control, initial = 0%
	PWM_LER = PWM_LER_MATCH0_LATCH | PWM_LER_MATCH2_LATCH; //latch new values for MR0 and MR2
	PWM_PCR = PWM_PCR_PWMENA2;                             //enable PWM2 in single edge control mode
	PWM_TCR = PWM_TCR_ENABLE | PWM_TCR_COUNTER_ENABLE;     //enable PWM and Counter

	/*
	 * connect signal PWM2 to pin P0.7
	 */

	PINSEL0 = (PINSEL0 & PINSEL0_PWM2_MASK) | PINSEL0_PWM2;
}

/*****************************************************************************
 *
 * Description:
 *    Update the duty cycle value of the PWM signal.
 *
 * Params:
 *    [in] dutyValue - the new duty cycle value in percent in increments
 *                     of 0.01% (i.e., 10% = 1000).
 *
 ****************************************************************************/
void pwmSetDutyPercent(unsigned int dutyValue)
{
  PWM_MR2 = (PWM_MR0 * dutyValue) / 10000;  //update duty cycle
  PWM_LER = 0x04;                           //latch new values for MR2
}
