/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Rob Jansen
 *
 * File:
 *    pwm.h
 *
 * Description:
 *    PWM defines and functions
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

/*
 * PWM Timer Control Register
 *
 * PWM_TCR_COUNTER_ENABLE - When one the PWM Timer Counter and PWM Prescale
 *                          counter are enabled for counting.
 * PWM_TCR_COUNTER_RESET  - When one the PWM Timer Counter and the PWM Prescale
 *                          counter are synchrounously reset on the next
 *                          positive edge of PCLK. The counters remain reset
 *                          until returned to 0.
 * PWM_TCR_ENABLE         - When one, PWM mode is enabled
 */

#define PWM_TCR_COUNTER_ENABLE (1<<0)
#define PWM_TCR_COUNTER_RESET  (1<<1)
#define PWM_TCR_ENABLE         (1<<3)

/*
 * PWM Match Control Register
 *
 * PWM_MCR_PWMMR0R - Reset on PWMMR0: The PWMTC will be reset if PWMMR0 matches it
 *
 * Note: only the used bits of this register are defined here
 */

#define PWM_MCR_PWMMR0R (1<<1)

/*
 * PWM Control Register bit values
 *
 * PWM_PCR_PWMSEL1..6 - Sets double edge mode for the PWM output
 *
 * PWM_PCR_PWMENA1..6 - Enables the PWM output
 */

#define PWM_PCR_PWMSEL2 (1<<2)
#define PWM_PCR_PWMSEL3 (1<<3)
#define PWM_PCR_PWMSEL4 (1<<4)
#define PWM_PCR_PWMSEL5 (1<<5)
#define PWM_PCR_PWMSEL6 (1<<6)
#define PWM_PCR_PWMENA1 (1<<9)
#define PWM_PCR_PWMENA2 (1<<10)
#define PWM_PCR_PWMENA3 (1<<11)
#define PWM_PCR_PWMENA4 (1<<12)
#define PWM_PCR_PWMENA5 (1<<13)
#define PWM_PCR_PWMENA6 (1<<14)

/*
 * PWM Latch Enable Register bit values
 * Set the corresponding bit to latch the new match values into the Match Register
 */
#define PWM_LER_MATCH0_LATCH (1<<0)
#define PWM_LER_MATCH1_LATCH (1<<1)
#define PWM_LER_MATCH2_LATCH (1<<2)
#define PWM_LER_MATCH3_LATCH (1<<3)
#define PWM_LER_MATCH4_LATCH (1<<4)
#define PWM_LER_MATCH5_LATCH (1<<5)
#define PWM_LER_MATCH6_LATCH (1<<6)

/*
 * PWM Pin Connect block
 *
 * PINSEL0_PWM1_MASK - Mask bits
 * PINSEL0_PWM1      - value bits
 *
 * Use the Mask and value to set the specified pin to the PWM
 * E.g: PINSEL0 = (PINSEL0 & PINSEL0_PWM1_MASK) | PINSEL0_PWM1
 *      will set P0.0 to PWM1 output
 *
 * See the User Manual for the specific LPC21xx controller for exact mapping
 * of IO pins to PWM outputs.
 */

#define PINSEL0_PWM1_MASK (~(3<< 0))
#define PINSEL0_PWM2_MASK (~(3<<14))
#define PINSEL0_PWM3_MASK (~(3<< 2))
#define PINSEL0_PWM4_MASK (~(3<<16))
#define PINSEL1_PWM5_MASK (~(3<<10))
#define PINSEL0_PWM6_MASK (~(3<<18))

#define PINSEL0_PWM1      (2<< 0)
#define PINSEL0_PWM2      (2<<14)
#define PINSEL0_PWM3      (2<< 2)
#define PINSEL0_PWM4      (2<<16)
#define PINSEL1_PWM5      (1<<10)
#define PINSEL0_PWM6      (2<<18)

void pwmInit(unsigned int initialFreqValue);
void pwmSetDutyPercent(unsigned int dutyValue);

