/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Rob Jansen
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application to test PCF8576 LCD driver.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include <pwm.h>
#include "startup/config.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

#define I2C_DISPLAY_ADDR  0x70	// Addr = 0 1 1 1.0 0 [SA0] [R/-W]


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);

/*****************************************************************************
 * Global data
 ****************************************************************************/

tU8 lcd_0[] = { 0x80, 0x80, 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01, 0x01, 0x01 };

/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (1000 * VPBDIV_FACTOR));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer

  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}

static void printError(tS8 error)
{
	if(error == I2C_CODE_OK)
	{
		printf("I2C_CODE_OK");
	}
	else if(error == I2C_CODE_ERROR)
	{
		printf("I2C_CODE_ERROR");
	}
	else if(error == I2C_CODE_EMPTY)
	{
		printf("I2C_CODE_EMPTY");
	}
	else if(error == I2C_CODE_BUSY)
	{
		printf("I2C_CODE_BUSY");
	}
}

/*
 * Convert data from human readable format into LCD data bytes.
 * Each byte of LCD data is presented as 2 rows of 4 pixels, one
 * byte representing the left half, the other byte the right half.
 * For the top two rows this is:
 *
 *    13/7 13/5 13/4 13/6  5/2  5/0  5/1  5/3
 *    13/3 13/1 13/0 13/2  5/6  5/4  5/5  5/7
 *
 *    (13,5 being the address, 0-7 the bits)
 *
 * This is followed by the following set of addresses:
 *
 *    14 4, 15 3, 16 2, 17 1, 18 0
 *
 * The human readable format contains sequential addresses, 0 being at
 * the top with bit 7..0 placed in 13/7 .. 5/3
 */

void convert(tU8 *input, tU8 *output)
{
	tU8 addr_left, addr_right;

	for(addr_left = 13; addr_left <=18; addr_left++)
	{
		addr_right = 18 - addr_left;

		output[addr_left]  = (*input & 0x80)    | (*input & 0x40)>>1 |
		                     (*input & 0x20)>>1 | (*input & 0x10)<<2;
		output[addr_right] = (*input & 0x08)>>2 | (*input & 0x04)>>2 |
		                     (*input & 0x02)    | (*input & 0x01)<<3;
		input++;
		output[addr_left]  |= (*input & 0x80)>>4 | (*input & 0x40)>>5 |
		                      (*input & 0x20)>>5 | (*input & 0x10)>>2;
		output[addr_right] |= (*input & 0x08)<<3 | (*input & 0x04)<<2 |
		                      (*input & 0x02)<<4 | (*input & 0x01)<<7;
		input++;
	}
}

	/*
	 * MODE SET = C 1 0 LP E B M1 M0
	 *            1 1 0 1  1 0 0  0  = 0xD8
	 *            1 1 0 1  1 1 0  0  = 0xDC
	 *
	 *     LP 0 = Normal Mode       *
	 *        1 = Power saving mode
	 *     E  0 = Display blank
	 *        1 = Display enabled   *
	 *     B  0 = 1/3 Bias
	 *        1 = 1/2 Bias          *
	 *     M* 00 = 4 Backplanes     *
	 *        11 = 3 Backplanes
	 *        10 = 2 Backplanes
	 *        01 = Static
	 *
	 */
#define MODE_SET_CMD 0xDC

	/*
	 * DEVICE SELECT = C 1 1 0 0 A2 A1 A0
	 *                 1 1 1 0 0 x  x  x  = 0xE0
	 *
	 *    A* = Subaddress
	 */
#define	DEVICE_SELECT_CMD(x) (0xE0 + (x & 7))

	/*
	 * BANK SELECT = C 1 1 1 1 0 I O
	 *               1 1 1 1 1 0 0 0 = 0xf8
	 *
	 *    I = selected input bank (0 for 3/4 BP)
	 *    O = selected output bank
	 *
	 * Since 4 BP mode is used, always set banks to 0
	 */
#define BANK_SELECT_CMD 0xF8

	/*
	 * BLINK = C 1 1 1 0 A BF1 BF0
	 *         1 1 1 1 0 0  0   0  = 0xF0
	 *
	 *    A    0 = Normal Blinking
	 *    BF* 00 = Off
	 *        01 = 2 Hz
	 *        10 = 1 Hz
	 *        11 = 0.5 Hz
	 */
#define BLINK_CMD(x) (0xF0 + x)
#define BLINK_OFF    0
#define BLINK_2HZ    1
#define BLINK_1HZ     2
#define BLINK_05HZ    3

	/*
	 * LOAD DATA POINTER = C 0 P5 P4 P3 P2 P1 P0
	 *                     0 0 x  x  x  x  x  x  = 0x80
	 *
	 *    P  * = Display RAM address [0..39]
	 *
	 *    C  0 = Last command
	 *           Always send this command as the last command
	 */
#define LOAD_DATA_POINTER_CMD(x) (0 + (x & 0x3F))

static void writeFullLcd(tU8 subaddress, tU8 *idata)
{
	tS8 retCode;

	tU8 sdata[] = { MODE_SET_CMD,
	                DEVICE_SELECT_CMD(subaddress),
	                BLINK_CMD(BLINK_OFF),
	                LOAD_DATA_POINTER_CMD(0),
	                0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69,
	                0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69
			      };


	printf("* Testing LCD with subaddress == %d\n", subaddress);

	if(idata != 0)
	{
		convert(idata, sdata);
	}

	retCode = i2cWrite(I2C_DISPLAY_ADDR, 0, 0, sdata, sizeof(sdata));

	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on %d=i2cWrite()\n",retCode);
		return;
	}
	i2cStop();

	printf(".\n");
}

static void lcdWrite(tU8 subaddress, tU8 address, tU8 data)
{
	tU8 counter = 0;
	tS8 retCode;

	retCode = i2cStart();
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on i2cStart()\n");
		return;
	}

	retCode = i2cPutCharAndWait(I2C_DISPLAY_ADDR);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending slave addr\n");
		return;
	}

	/*
	 * DEVICE SELECT = C 1 1 0 0 A2 A1 A0
	 *                 1 1 1 0 0 x  x  x  = 0xE0
	 *
	 *    A* = Subaddress
	 */
	retCode = i2cPutCharAndWait(0xe0 + subaddress);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending DEVICE SELECT [%d] command\n", subaddress);
		return;
	}

	/*
	 * LOAD DATA POINTER = C 0 P5 P4 P3 P2 P1 P0
	 *                     1 0 0  0  0  0  0  0  = 0x80
	 *
	 *    P* = Display RAM address [0..39]
	 */
	retCode = i2cPutCharAndWait(address);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending LOAD DATA POINTER command\n");
		return;
	}

	/*
	 * Now send Databyte(s), 20 bytes for 20 * 2 * 4 bits
	 */
	retCode = i2cPutCharAndWait(data);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending data, counter = %d\n", counter);
	}

	i2cStop();
	printf(".\n");

}

/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function.
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/

int main(void)
{
	tU8 counter = 0;
	tU8 address, bit;
	char c;

	//initialize printf()-functionality
	eaInit();

	//print welcome message
	printf("\n*********************************************************");
	printf("\n*");
	printf("\n* (C) 2005 Rob Jansen");
	printf("\n*");
	printf("\n* Welcome to the LCD test program, using");
	printf("\n* the i2c library to write to a PCF8576 LCD Driver.");
	printf("\n*");
	printf("\n*********************************************************\n\n");

	//initialice i2c communication
	i2cInit(100000);

	//initialize PWM unit
	pwmInit(500);         /* 500 Hz              */
	pwmSetDutyPercent(7); /* Duty cycle = 0.07 % */


	//enter forever loop...
	while(1)
	{
		printf("\n* Enter [0-7,d,w]");
		c = consolGetCh();
		if((c>='0') && (c<='7'))
		{
			writeFullLcd(c-'0', lcd_0);
			for(counter=0; counter<12; counter++)
				lcd_0[counter] = (lcd_0[counter]&1)<<7 | lcd_0[counter] >>1;
		}
		else if (c == 'd')
		{
			printf("\n* Enter dutycycle in 0.01%%:");
			counter = consolGetIntNum();
			pwmSetDutyPercent(counter);
		}
		else if (c == 'w')
		{
			printf("\n* Enter RAM address:");
			address = consolGetIntNum();
			printf("\n* Enter bit:");
			bit = consolGetIntNum();
			lcdWrite(0, address, 1<<bit);
		}
		else
		{
			printf("\n* Invalid input\n");
		}
	}

	return 0;
}
