/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Rob Jansen
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application to test PCF8576 LCD driver.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include <pwm.h>
#include "startup/config.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

#define I2C_DISPLAY_ADDR  0x70	// Addr = 0 1 1 1.0 0 [SA0] [R/-W]


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (1000 * VPBDIV_FACTOR));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer

  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}

static void printError(tS8 error)
{
	if(error == I2C_CODE_OK)
	{
		printf("I2C_CODE_OK");
	}
	else if(error == I2C_CODE_ERROR)
	{
		printf("I2C_CODE_ERROR");
	}
	else if(error == I2C_CODE_EMPTY)
	{
		printf("I2C_CODE_EMPTY");
	}
	else if(error == I2C_CODE_BUSY)
	{
		printf("I2C_CODE_BUSY");
	}
}

static void writeToSubaddress(tU8 subaddress)
{
	tU8 counter = 0;
	tS8 retCode;

	printf("* Testing LCD with subaddress == %d\n", subaddress);

	retCode = i2cStart();
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on i2cStart()\n");
		return;
	}

	retCode = i2cPutCharAndWait(I2C_DISPLAY_ADDR);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending slave addr\n");
		return;
	}

	/*
	 * MODE SET = C 1 0 LP E B M1 M0
	 *            1 1 0 1  1 0 0  0  = 0xD8
	 *            1 1 0 1  1 1 0  0  = 0xDC
	 *
	 *     LP 0 = Normal Mode
	 *        1 = Power saving mode
	 *     E  0 = Display blank
	 *        1 = Display enabled
	 *     B  0 = 1/3 Bias
	 *        1 = 1/2 Bias
	 *     M* 00 = 4 Backplanes
	 *        11 = 3 Backplanes
	 *        10 = 2 Backplanes
	 *        01 = Static
	 *
	 */
	retCode = i2cPutCharAndWait(0xdc);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending MODE SET command\n");
		return;
	}

	/*
	 * LOAD DATA POINTER = C 0 P5 P4 P3 P2 P1 P0
	 *                     1 0 0  0  0  0  0  0  = 0x80
	 *
	 *    P* = Display RAM address [0..39]
	 */
	retCode = i2cPutCharAndWait(0x80);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending LOAD DATA POINTER command\n");
		return;
	}

	/*
	 * DEVICE SELECT = C 1 1 0 0 A2 A1 A0
	 *                 1 1 1 0 0 x  x  x  = 0xE0
	 *
	 *    A* = Subaddress
	 */
	retCode = i2cPutCharAndWait(0xe0 + subaddress);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending DEVICE SELECT [%d] command\n", subaddress);
		return;
	}
	subaddress++;    // increment subaddress
	subaddress &= 7;

	/*
	 * BANK SELECT = C 1 1 1 1 0 I O
	 *               1 1 1 1 1 0 0 0 = 0xf8
	 *
	 *    I = selected input bank (0 for 3/4 BP)
	 *    O = selected output bank
	 */
	retCode = i2cPutCharAndWait(0xf8);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending BANK SELECT command\n");
		return;
	}

	/*
	 * BLINK = C 1 1 1 0 A BF1 BF0
	 *         0 1 1 1 0 0  0   0  = 0x70
	 *
	 *    A    0 = Normal Blinking
	 *    BF* 00 = Off
	 *        01 = 2 Hz
	 *        10 = 1 Hz
	 *        11 = 0.5 Hz
	 */
	retCode = i2cPutCharAndWait(0x70);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending BLINK command\n");
		return;
	}

	/*
	 * Now send Databytes, 20 bytes for 20 * 2 * 4 bits
	 */
	for(counter=0; counter<20; counter++)
	{
		retCode = i2cPutCharAndWait(0x69);
		if(retCode != I2C_CODE_OK)
		{
			break;
		}
	}

	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending data, counter = %d\n", counter);
	}

	i2cStop();
	printf(".\n");

}

static void lcdWrite(tU8 subaddress, tU8 address, tU8 data)
{
	tU8 counter = 0;
	tS8 retCode;

	retCode = i2cStart();
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on i2cStart()\n");
		return;
	}

	retCode = i2cPutCharAndWait(I2C_DISPLAY_ADDR);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending slave addr\n");
		return;
	}

	/*
	 * DEVICE SELECT = C 1 1 0 0 A2 A1 A0
	 *                 1 1 1 0 0 x  x  x  = 0xE0
	 *
	 *    A* = Subaddress
	 */
	retCode = i2cPutCharAndWait(0xe0 + subaddress);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending DEVICE SELECT [%d] command\n", subaddress);
		return;
	}

	/*
	 * LOAD DATA POINTER = C 0 P5 P4 P3 P2 P1 P0
	 *                     1 0 0  0  0  0  0  0  = 0x80
	 *
	 *    P* = Display RAM address [0..39]
	 */
	retCode = i2cPutCharAndWait(address);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending LOAD DATA POINTER command\n");
		return;
	}

	/*
	 * Now send Databyte(s), 20 bytes for 20 * 2 * 4 bits
	 */
	retCode = i2cPutCharAndWait(data);
	if(retCode != I2C_CODE_OK)
	{
		printError(retCode);
		printf(" returned on sending data, counter = %d\n", counter);
	}

	i2cStop();
	printf(".\n");

}

/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function.
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
	tU32 freq;
	tU8 counter = 0;
	tU8 address, bit;
	char c;

	//initialize printf()-functionality
	eaInit();

	//print welcome message
	printf("\n*********************************************************");
	printf("\n*");
	printf("\n* (C) 2005 Rob Jansen");
	printf("\n*");
	printf("\n* Welcome to the LCD test program, using");
	printf("\n* the i2c library to write to a PCF8576 LCD Driver.");
	printf("\n*");
	printf("\n*********************************************************\n\n");

	//initialice i2c communication
	i2cInit(100000);

	//set frequency to  (50 kHz)
	freq = ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (VPBDIV_FACTOR * 50000));

	//initialize PWM unit
	pwmInit(500);
	pwmSetDutyPercent(7); /* Duty cycle = 0.07 % */


	//enter forever loop...
	while(1)
	{
		printf("\n* Enter [0-7,d,w]");
		c = consolGetCh();
		if((c>='0') && (c<='7'))
		{
			writeToSubaddress(c-'0');
		}
		else if (c == 'd')
		{
			printf("\n* Enter dutycycle in 0.01%%:");
			counter = consolGetIntNum();
			pwmSetDutyPercent(counter);
		}
		else if (c == 'w')
		{
			printf("\n* Enter RAM address:");
			address = consolGetIntNum();
			printf("\n* Enter bit:");
			bit = consolGetIntNum();
			lcdWrite(0, address, 1<<bit);
		}
		else
		{
			printf("\n* Invalid input\n");
		}
	}

	return 0;
}
